##
## widget.py
## Login : <uli@pu.smp.net>
## Started on  Tue Sep 14 11:00:16 2010 Uli Fouquet
## $Id$
## 
## Copyright (C) 2010 Uli Fouquet
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
"""Image file widgets.
"""
from waeup.sirp.image import WAeUPImageFile
from hurry.file.browser.widget import (
    FileWidgetBase, EncodingFileWidget, DownloadWidget,
    )
from zope.app.form.browser.textwidgets import escape
from zope.app.form.browser.widget import renderElement
from zope.app.form.interfaces import ConversionError


class EncodingImageFileWidget(EncodingFileWidget):
    """A hurry.file widget that stores images.

    This is an upload widget suitable for edit forms and the like.
    """
    #: We can set a basename for the image to be rendered.
    #: This will result in <img> links like <img src='.../<basename>.jpg' />
    image_basename = None

    def _toFieldValue(self, (input, file_id)):
       # we got no file upload input
        if not input:
            # if we got a file_id, then retrieve file and return it
            if file_id:
                return self._retrieveFile(file_id)
            # no file upload input nor file id, so return missing value
            return self.context.missing_value
        # read in file from input
        try:
            seek = input.seek
            read = input.read
        except AttributeError, e:
            raise ConversionError('Form input is not a file object', e)

        seek(0)
        data = read()

        if data:
            return WAeUPImageFile(input.filename, data)
        else:
            return self.context.missing_value

    def __call__(self):
        # The base widget renders the actual image as filename. We
        # want the image itself displayed as well.
        value = self._getFormValue()
        result = u''
        if value:
            filename = value.filename
            if self.image_basename is not None:
                filename = self.image_basename + os.path.splitext(filename)[-1]
            result = renderElement(
                'img',
                src=filename,
                )
            result += renderElement('br')
        # Append the rendering as delivered from the base widget.
        result += super(EncodingImageFileWidget, self).__call__()
        return result

    def _storeFile(self, file_upload):
        # filenames are normally in unicode encoding, while the contents
        # are byte streams. We turn the filename into a bytestream.
        data = '%s\n%s' % (str(file_upload.filename), file_upload.read())
        return data.encode('base64')[:-1]

    def _retrieveFile(self, file_id):
        data = file_id.decode('base64')
        filename, filedata = data.split('\n', 1)
        return WAeUPImageFile(filename, filedata)

class ThumbnailWidget(DownloadWidget):
    """An image file widget that displays the data as thumbnail.

    XXX: give some reason to name this a _thumbnail_ widget.
    """

    def __call__(self):
        if self._renderedValueSet():
            value = self._data
        else:
            value = self.context.default
        if value == self.context.missing_value:
            return renderElement(
                u'div',
                contents=u'Download not available')
        filename = escape(value.filename)
        return renderElement(
            u'img',
            src=filename,
            contents=None)
